@@ -1,11 +1,11 @@
 use strict;
 use warnings;
-package JSON::Schema::Modern; # git description: v0.623-10-g10919909
+package JSON::Schema::Modern; # git description: v0.624-9-g11d40e29
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Validate data against a schema using a JSON Schema
 # KEYWORDS: JSON Schema validator data validation structure specification
 
-our $VERSION = '0.624';
+our $VERSION = '0.625';
 
 use 5.020;  # for fc, unicode_strings features
 use Moo;
@@ -24,7 +24,6 @@ use Carp qw(croak carp);
 use List::Util 1.55 qw(pairs first uniqint pairmap uniq min);
 use if "$]" < 5.041010, 'List::Util' => 'any';
 use if "$]" >= 5.041010, experimental => 'keyword_any';
-use Ref::Util 0.100 qw(is_ref is_plain_hashref);
 use builtin::compat qw(refaddr load_module);
 use Mojo::URL;
 use Safe::Isa;
@@ -38,7 +37,7 @@ use Feature::Compat::Try;
 use JSON::Schema::Modern::Error;
 use JSON::Schema::Modern::Result;
 use JSON::Schema::Modern::Document;
-use JSON::Schema::Modern::Utilities qw(get_type canonical_uri E abort annotate_self jsonp is_type assert_uri local_annotations is_schema json_pointer_type canonical_uri_type);
+use JSON::Schema::Modern::Utilities qw(get_type canonical_uri E abort annotate_self jsonp is_type assert_uri local_annotations is_schema json_pointer_type canonical_uri_type load_cached_document);
 use namespace::clean;
 
 our @CARP_NOT = qw(
@@ -124,7 +123,7 @@ sub _get_format_validation ($self, $format) { ($self->{_format_validations}//{})
 sub add_format_validation ($self, $format, $definition) {
   return if exists(($self->{_format_validations}//{})->{$format});
 
-  $definition = { type => 'string', sub => $definition } if not is_plain_hashref($definition);
+  $definition = { type => 'string', sub => $definition } if ref $definition ne 'HASH';
   $format_type->({ $format => $definition });
 
   # all core formats are of type string (so far); changing type of custom format is permitted
@@ -145,7 +144,7 @@ around BUILDARGS => sub ($orig, $class, @args) {
     if $args->{collect_annotations} and ($args->{specification_version}//'') =~ /^draft[467]$/;
 
   $args->{format_validations} = +{
-    map +($_->[0] => is_plain_hashref($_->[1]) ? $_->[1] : +{ type => 'string', sub => $_->[1] }),
+    map +($_->[0] => ref $_->[1] eq 'HASH' ? $_->[1] : +{ type => 'string', sub => $_->[1] }),
       pairs $args->{format_validations}->%*
   } if $args->{format_validations};
 
@@ -219,30 +218,36 @@ sub add_document {
     exception => 1,
   ) if $document->has_errors;
 
+  if (not length $base_uri){
+    foreach my $res_pair ($document->resource_pairs) {
+      my ($uri_string, $doc_resource) = @$res_pair;
+
+      # this might croak if there are duplicates or malformed entries.
+      $self->_add_resource($uri_string => +{ $doc_resource->%*, document => $document });
+    }
+
+    return $document;
+  }
+
   my @root; # uri_string => resource hash of the resource at path ''
 
   # document resources are added after resolving each resource against our provided base uri
   foreach my $res_pair ($document->resource_pairs) {
     my ($uri_string, $doc_resource) = @$res_pair;
-    $uri_string = Mojo::URL->new($uri_string)->to_abs($base_uri)->to_string if length $base_uri;
+    $uri_string = Mojo::URL->new($uri_string)->to_abs($base_uri)->to_string;
 
     my $new_resource = {
+      canonical_uri => Mojo::URL->new($doc_resource->{canonical_uri})->to_abs($base_uri),
       $doc_resource->%{qw(path specification_version vocabularies)},
       document => $document,
     };
 
-    $new_resource->{canonical_uri} = length $base_uri
-      ? Mojo::URL->new($doc_resource->{canonical_uri})->to_abs($base_uri)
-      : $doc_resource->{canonical_uri};
-
     foreach my $anchor (keys (($doc_resource->{anchors}//{})->%*)) {
       use autovivification 'store';
       $new_resource->{anchors}{$anchor} = {
         $doc_resource->{anchors}{$anchor}->%{path},
         (map +($_->[1] ? @$_ : ()), [ $doc_resource->{anchors}{$anchor}->%{dynamic} ]),
-        canonical_uri => length $base_uri
-          ? Mojo::URL->new($doc_resource->{anchors}{$anchor}{canonical_uri})->to_abs($base_uri)
-          : $doc_resource->{anchors}{$anchor}{canonical_uri},
+        canonical_uri => Mojo::URL->new($doc_resource->{anchors}{$anchor}{canonical_uri})->to_abs($base_uri),
       };
     }
 
@@ -252,7 +257,7 @@ sub add_document {
   }
 
   # associate the root resource with the base uri we were provided, if it does not already exist
-  $self->_add_resource($base_uri.'' => $root[1]) if length $base_uri and $root[0] ne $base_uri;
+  $self->_add_resource($base_uri.'' => $root[1]) if $root[0] ne $base_uri;
 
   return $document;
 }
@@ -334,7 +339,7 @@ sub traverse ($self, $schema_reference, $config_override = {}) {
     # a subsequent "$schema" keyword can still change these values, and it is always processed
     # first, so the override is skipped if the keyword exists in the schema
     $state->{metaschema_uri} =
-      (is_plain_hashref($schema_reference) && exists $schema_reference->{'$schema'} ? undef
+      (ref $schema_reference eq 'HASH' && exists $schema_reference->{'$schema'} ? undef
         : $config_override->{metaschema_uri}) // $self->METASCHEMA_URIS->{$spec_version};
 
     if (my $metaschema_info = $self->_get_metaschema_vocabulary_classes($state->{metaschema_uri})) {
@@ -396,7 +401,7 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
       # traverse is called via add_schema -> ::Document->new -> ::Document->BUILD
       $schema_reference = $self->add_schema($schema_reference)->canonical_uri;
     }
-    elsif (is_ref($schema_reference) and not $schema_reference->$_isa('Mojo::URL')) {
+    elsif (ref $schema_reference and not $schema_reference->$_isa('Mojo::URL')) {
       abort($state, 'invalid schema type: %s', get_type($schema_reference));
     }
 
@@ -475,7 +480,7 @@ sub evaluate ($self, $data, $schema_reference, $config_override = {}) {
 sub validate_schema ($self, $schema, $config_override = {}) {
   croak 'validate_schema called in void context' if not defined wantarray;
 
-  my $metaschema_uri = is_plain_hashref($schema) && $schema->{'$schema'} ? $schema->{'$schema'}
+  my $metaschema_uri = ref $schema eq 'HASH' && $schema->{'$schema'} ? $schema->{'$schema'}
     : $self->METASCHEMA_URIS->{$self->specification_version // $self->SPECIFICATION_VERSION_DEFAULT};
 
   my $result = $self->evaluate($schema, $metaschema_uri,
@@ -498,11 +503,11 @@ sub get ($self, $uri_reference) {
   if (wantarray) {
     my $schema_info = $self->_fetch_from_uri($uri_reference);
     return if not $schema_info;
-    my $subschema = is_ref($schema_info->{schema}) ? dclone($schema_info->{schema}) : $schema_info->{schema};
+    my $subschema = ref $schema_info->{schema} ? dclone($schema_info->{schema}) : $schema_info->{schema};
     return ($subschema, $schema_info->{canonical_uri});
   }
   else {  # abridged version of _fetch_from_uri
-    $uri_reference = Mojo::URL->new($uri_reference) if not is_ref($uri_reference);
+    $uri_reference = Mojo::URL->new($uri_reference) if not ref $uri_reference;
     my $fragment = $uri_reference->fragment;
     my $resource = $self->_get_or_load_resource($uri_reference->clone->fragment(undef));
     return if not $resource;
@@ -515,7 +520,7 @@ sub get ($self, $uri_reference) {
       return if not my $subresource = ($resource->{anchors}//{})->{$fragment};
       $schema = $resource->{document}->get($subresource->{path});
     }
-    return is_ref($schema) ? dclone($schema) : $schema;
+    return ref $schema ? dclone($schema) : $schema;
   }
 }
 
@@ -713,7 +718,7 @@ sub _eval_subschema ($self, $data, $schema, $state) {
   # We also set it when _strict_schema_data is set, but only for object data instances.
   $state->{collect_annotations} |=
     0+(exists $schema->{unevaluatedItems} || exists $schema->{unevaluatedProperties}
-      || !!$state->{seen_data_properties} && (my $is_object_data = is_plain_hashref($data)));
+      || !!$state->{seen_data_properties} && (my $is_object_data = ref $data eq 'HASH'));
 
   # in order to collect annotations for unevaluated* keywords, we sometimes need to ignore the
   # suggestion to short_circuit evaluation at this scope (but lower scopes are still fine)
@@ -864,25 +869,28 @@ sub _resource_pairs { pairs(($_[0]->{_resource_index}//{})->%*) }
 
 sub _add_resource ($self, @kvs) {
   foreach my $pair (sort { $a->[0] cmp $b->[0] } pairs @kvs) {
-    my ($key, $value) = @$pair;
+    my ($canonical_uri, $resource) = @$pair;
 
-    if (my $existing = $self->_get_resource($key)) {
+    if (my $existing = $self->_get_resource($canonical_uri)) {
       # we allow overwriting canonical_uri = '' to allow for ad hoc evaluation of schemas that
       # lack all identifiers altogether, but preserve other resources from the original document
-      if ($key ne '') {
-        next if $existing->{path} eq $value->{path}
-          and $existing->{canonical_uri} eq $value->{canonical_uri}
-          and $existing->{specification_version} eq $value->{specification_version}
-          and refaddr($existing->{document}) == refaddr($value->{document});
-        croak 'uri "'.$key.'" conflicts with an existing schema resource';
+      if ($canonical_uri ne '') {
+        my @diffs = (
+          ($existing->{path} eq $resource->{path} ? () : 'path'),
+          ($existing->{canonical_uri} eq $resource->{canonical_uri} ? () : 'canonical_uri'),
+          ($existing->{specification_version} eq $resource->{specification_version} ? () : 'specification_version'),
+          (refaddr($existing->{document}) == refaddr($resource->{document}) ? () : 'refaddr'));
+        next if not @diffs;
+        croak 'uri "'.$canonical_uri.'" conflicts with an existing schema resource: documents differ by ',
+          join(', ', @diffs);
       }
     }
-    elsif ($self->CACHED_METASCHEMAS->{$key}) {
-      croak 'uri "'.$key.'" conflicts with an existing meta-schema resource';
+    elsif (JSON::Schema::Modern::Utilities::get_schema_filename($canonical_uri)) {
+      croak 'uri "'.$canonical_uri.'" conflicts with an existing cached schema resource';
     }
 
     use autovivification 'store';
-    $self->{_resource_index}{$resource_key_type->($key)} = $resource_type->($value);
+    $self->{_resource_index}{$resource_key_type->($canonical_uri)} = $resource_type->($resource);
   }
 }
 
@@ -1022,7 +1030,8 @@ use constant METASCHEMA_URIS => {
   'draft4' => 'http://json-schema.org/draft-04/schema',
 };
 
-use constant CACHED_METASCHEMAS => {
+# for internal use only. files are under share/
+use constant _CACHED_METASCHEMAS => {
   'https://json-schema.org/draft/2020-12/meta/applicator'     => 'draft2020-12/meta/applicator.json',
   'https://json-schema.org/draft/2020-12/meta/content'        => 'draft2020-12/meta/content.json',
   'https://json-schema.org/draft/2020-12/meta/core'           => 'draft2020-12/meta/core.json',
@@ -1052,35 +1061,18 @@ use constant CACHED_METASCHEMAS => {
 # simple runtime-wide cache of metaschema document objects that are sourced from disk
 my $metaschema_cache = {};
 
+{
+  my $share_dir = dist_dir('JSON-Schema-Modern');
+  JSON::Schema::Modern::Utilities::register_schema($_, $share_dir.'/'._CACHED_METASCHEMAS->{$_})
+    foreach keys _CACHED_METASCHEMAS->%*;
+}
+
 # returns the same as _get_resource
 sub _get_or_load_resource ($self, $uri) {
   my $resource = $self->_get_resource($uri);
   return $resource if $resource;
 
-  if (my $local_filename = $self->CACHED_METASCHEMAS->{$uri}) {
-    my $document;
-    if (not $document = $metaschema_cache->{$local_filename}) {
-      my $file = path(dist_dir('JSON-Schema-Modern'), $local_filename);
-      my $schema = $self->_json_decoder->decode($file->slurp);
-      my $_document = JSON::Schema::Modern::Document->new(schema => $schema, evaluator => $self);
-
-      # this should be caught by the try/catch in evaluate()
-      die JSON::Schema::Modern::Result->new(
-        output_format => $self->output_format,
-        valid => 0,
-        errors => [ $_document->errors ],
-        exception => 1,
-      ) if $_document->has_errors;
-
-      $metaschema_cache->{$local_filename} = $document = $_document;
-    }
-
-    # we have already performed the appropriate collision checks, so we bypass them here
-    $self->_add_resources_unsafe(
-      map +($_->[0] => +{ $_->[1]->%*, document => $document }),
-        $document->resource_pairs
-    );
-
+  if (my $document = load_cached_document($self, $uri)) {
     return $self->_get_resource($uri);
   }
 
@@ -1297,7 +1289,7 @@ JSON::Schema::Modern - Validate data against a schema using a JSON Schema
 
 =head1 VERSION
 
-version 0.624
+version 0.625
 
 =head1 SYNOPSIS
 
@@ -1505,7 +1497,7 @@ Defaults to false.
 =head1 METHODS
 
 =for Pod::Coverage BUILDARGS FREEZE THAW
-CACHED_METASCHEMAS METASCHEMA_URIS SPECIFICATION_VERSIONS_SUPPORTED SPECIFICATION_VERSION_DEFAULT
+METASCHEMA_URIS SPECIFICATION_VERSIONS_SUPPORTED SPECIFICATION_VERSION_DEFAULT
 
 =head2 evaluate_json_string
 
