@@ -4,7 +4,7 @@ package JSON::Schema::Modern::Vocabulary::Content;
 # vim: set ts=8 sts=2 sw=2 tw=100 et :
 # ABSTRACT: Implementation of the JSON Schema Content vocabulary
 
-our $VERSION = '0.624';
+our $VERSION = '0.625';
 
 use 5.020;
 use Moo;
@@ -19,7 +19,6 @@ no if "$]" >= 5.033006, feature => 'bareword_filehandles';
 no if "$]" >= 5.041009, feature => 'smartmatch';
 no feature 'switch';
 use Storable 'dclone';
-use Ref::Util 0.100 'is_ref';
 use Feature::Compat::Try;
 use JSON::Schema::Modern::Utilities qw(is_type A assert_keyword_type E abort);
 use namespace::clean;
@@ -107,7 +106,7 @@ sub _eval_keyword_contentSchema ($class, $data, $schema, $state) {
         { %$state, keyword_path => $state->{keyword_path}.'/contentSchema' });
   }
 
-  return A($state, is_ref($schema->{contentSchema}) ? dclone($schema->{contentSchema}) : $schema->{contentSchema});
+  return A($state, ref $schema->{contentSchema} ? dclone($schema->{contentSchema}) : $schema->{contentSchema});
 }
 
 1;
@@ -124,7 +123,7 @@ JSON::Schema::Modern::Vocabulary::Content - Implementation of the JSON Schema Co
 
 =head1 VERSION
 
-version 0.624
+version 0.625
 
 =head1 DESCRIPTION
 
