@@ -15,11 +15,11 @@ use open ':std', ':encoding(UTF-8)'; # force stdin, stdout, stderr into utf8
 use Safe::Isa;
 use Feature::Compat::Try;
 use Mojo::File 'path';
+use builtin::compat 'blessed';
 
 use if $ENV{AUTHOR_TESTING}, 'Test2::Warnings' => ':fail_on_warning'; # hooks into done_testing unless overridden
 use Test::JSON::Schema::Acceptance 1.029;
 use Test::Memory::Cycle;
-use Test::File::ShareDir -share => { -dist => { 'JSON-Schema-Modern' => 'share' } };
 use JSON::Schema::Modern;
 
 # supports options:
@@ -50,7 +50,7 @@ sub acceptance_tests (%options) {
       test_dir => $accepter->test_dir->child($options{acceptance}{test_subdir}))
     if not $ENV{TEST_DIR} and $options{acceptance}{test_subdir};
 
-  $note->('Using JSON decoder: ', builtin::compat::blessed($accepter->_json_serializer), ' ', $accepter->_json_serializer->VERSION);
+  $note->('Using JSON decoder: ', blessed($accepter->_json_serializer), ' ', $accepter->_json_serializer->VERSION);
   $note->('');
 
   my $js = JSON::Schema::Modern->new($options{evaluator}->%*);
